import * as React from 'react';
import {observer} from "mobx-react-lite";
import {FormProps, MaterialFAIcon, ModelCheckbox, ModelForm, ModelInput, PaddedPaper, SelectField} from "@rx-suite/ui";
import {makeStyles} from "@material-ui/core/styles";
import {Grid, Typography} from "@material-ui/core";
import {t} from "@lingui/macro";
import {faArrowsAltH, faArrowsAltV} from "@rx-suite/pro-solid-svg-icons";
import {TemplateModel} from "../../stores/TemplateStore";

const useStyles = makeStyles(theme => ({
    widthFieldWrapper: {
        paddingLeft: theme.spacing(2),
        paddingRight: theme.spacing(1)
    },
    mainPageLayoutWrapper: {
        paddingLeft: theme.spacing(1),
        paddingRight: theme.spacing(1),
        paddingBottom: theme.spacing(2),
        alignItems: "stretch"
    },
    fakePage: {
        borderStyle: 'solid',
        borderColor: theme.palette.grey[400],
        width: '100%',
        height: '100%',
        padding: theme.spacing(2)
    },
    fakePageContents: {
        borderStyle: 'solid',
        borderColor: theme.palette.grey[400],
        width: '100%',
        height: '100%',
        padding: theme.spacing(2)
    },
    sidebarWrapper: {
        display: 'flex',
        flexDirection: 'column',
        alignItems: 'center',
        height: '100%',
        paddingLeft: theme.spacing(1),
    },
    heightSpacer: {
        flexGrow: 1
    }
}));


export const TemplateForm: React.FunctionComponent<FormProps> = observer(({model, readOnly}) => {
    const classes = useStyles();
    const paperOptions = [];
    if (model.viewModel.paperType === 'custom') {
        paperOptions.push({
            label: t`Custom`,
            value: 'custom'
        });
    }
    (model.paperTypes as { label: string; value: string; }[]).forEach((type) => {
        paperOptions.push({
            label: type.label,
            value: type.value
        });
    });
    const paperTypeLabel = t`Page Setup`;
    return (
        <PaddedPaper>
            <ModelForm model={model} readOnly={readOnly}>
                <ModelInput name={"name"}/>
                <ModelInput name={"desc"} multiline rowsMax={4}/>
                <Typography variant={'h4'}>{t`Page Setup`}</Typography>
                <Grid container className={classes.widthFieldWrapper} spacing={2} alignItems={'flex-end'}>
                    <Grid item xs={2}>
                        <SelectField value={model.viewModel.paperType}
                                     label={paperTypeLabel}
                                     hint={{
                                         title: paperTypeLabel,
                                         text: t`Pick from standard paper dimensions`
                                     }}
                                     onChange={(_, value) => {
                                         (model as TemplateModel).handleSetPaperType(value);
                                     }}
                                     options={paperOptions}
                                     disabled={readOnly}/>
                    </Grid>
                    <Grid item xs={2}>
                        <ModelCheckbox name={'landscape'}/>
                    </Grid>
                    <Grid item xs={7}><ModelInput name={"width"}
                                                  type={"number"} inputProps={{min: 0, step: 0.01}}
                                                  label={`${model.label('width')} (${model.viewModel.length_unit})`}/>
                    </Grid>
                    <Grid item xs={1}><MaterialFAIcon icon={faArrowsAltH}/></Grid>
                </Grid>
                <Grid container className={classes.mainPageLayoutWrapper} spacing={2} alignItems={'center'}>
                    <Grid item xs={2}>
                        <div className={classes.sidebarWrapper}>
                            <SelectField value={model.viewModel.length_unit}
                                         label={model.label('length_unit')}
                                         hint={{
                                             title: model.label('length_unit'),
                                             text: model.attributeDescription('length_unit')
                                         }}
                                         onChange={(_, value) => {
                                             (model as TemplateModel).setUnitType(value);
                                         }}
                                         options={model.units as { label: string, value: string }[]}
                                         disabled={readOnly}/>
                            <div className={classes.heightSpacer}/>
                            <ModelInput name={"height"}
                                        type={"number"} inputProps={{min: 0, step: 0.01}}
                                        label={`${model.label('height')} (${model.viewModel.length_unit})`}/>
                            <div className={classes.heightSpacer}/>
                            <MaterialFAIcon icon={faArrowsAltV}/>
                        </div>
                    </Grid>
                    <Grid item xs={9}>
                        <div className={classes.fakePage}>
                            <Grid container spacing={2} alignItems={'center'}>
                                {/*r1*/}
                                <Grid item xs={2}/>
                                <Grid item xs={8}>
                                    <ModelInput name={"margin_top"}
                                                type={"number"} inputProps={{min: 0, step: 0.01}}
                                                label={`${model.label('margin_top')} (${model.viewModel.length_unit})`}/>
                                </Grid>
                                <Grid item xs={2}/>
                                {/*r2*/}
                                <Grid item xs={2}><ModelInput name={"margin_left"}
                                                              type={"number"} inputProps={{min: 0, step: 0.01}}
                                                              label={`${model.label('margin_left')} (${model.viewModel.length_unit})`}/></Grid>
                                <Grid item xs={8}>
                                    <div className={classes.fakePageContents}>
                                        Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor
                                        incididunt ut labore et dolore magna aliqua. Id aliquet lectus proin nibh nisl
                                        condimentum. Vitae purus faucibus ornare suspendisse sed nisi lacus sed. Dolor
                                        morbi non arcu risus quis varius quam quisque id. Dis parturient montes nascetur
                                        ridiculus mus mauris. Magna fringilla urna porttitor rhoncus dolor purus. Odio
                                        eu feugiat pretium nibh ipsum consequat nisl vel pretium. At ultrices mi tempus
                                        imperdiet nulla malesuada pellentesque elit eget. Eget nunc scelerisque viverra
                                        mauris in aliquam sem fringilla. In iaculis nunc sed augue lacus viverra.
                                        Fermentum et sollicitudin ac orci phasellus egestas. Vitae sapien pellentesque
                                        habitant morbi tristique senectus et netus et. Duis ultricies lacus sed turpis
                                        tincidunt id. Nunc congue nisi vitae suscipit tellus. Enim eu turpis egestas
                                        pretium aenean pharetra magna ac. Adipiscing elit pellentesque habitant morbi
                                        tristique senectus et netus. Amet cursus sit amet dictum sit amet justo donec
                                        enim. Est ultricies integer quis auctor elit sed vulputate.
                                    </div>
                                </Grid>
                                <Grid item xs={2}><ModelInput name={"margin_right"}
                                                              type={"number"} inputProps={{min: 0, step: 0.01}}
                                                              label={`${model.label('margin_right')} (${model.viewModel.length_unit})`}/></Grid>
                                {/*r3*/}
                                <Grid item xs={2}/>
                                <Grid item xs={8}>
                                    <ModelInput name={"margin_bottom"}
                                                type={"number"} inputProps={{min: 0, step: 0.01}}
                                                label={`${model.label('margin_bottom')} (${model.viewModel.length_unit})`}/>
                                </Grid>
                                <Grid item xs={2}/>
                            </Grid>
                        </div>
                    </Grid>
                    <Grid item xs={1}></Grid>
                </Grid>
            </ModelForm>
        </PaddedPaper>
    )
});
