import {attribute, model, Model, Transport} from '@rx-suite/data';
import {t} from "@lingui/macro";
import * as yup from "yup";
import {computed} from "mobx";
import FileSaver from "file-saver";

class PdfResultTransport<T extends Model> extends Transport<T> {
    async run(parameters: any) {
        const resp = await this.fetch(`/run`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({
                ...parameters,
                appId: localStorage.getItem('appId')
            })
        });
        return resp.json();
    }

    async rerun(id: string) {
        const resp = await this.fetch(`/${id}/rerun`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            }
        });
        return resp.json();
    }

    async download(link: string) {
        const resp = await PdfResultTransport.fetch(link, {method: 'GET'});
        if (resp.ok) {
            return resp.blob();
        }
    }
}

@model({
    transportLayer: new PdfResultTransport("/api/pdf_results", "pdf_result"),
    routingInfo: {
        list: "/results",
        read: "/results/:id/read",
    }
})
export class PdfResultModel extends Model {
    constructor() {
        super();
        super.setup();
    }

    @attribute({
        label: t`Name`,
        validation: yup.string()
    })
    name = '';

    @attribute({
        label: t`Notes`,
        validation: yup.string().nullable()
    })
    notes = '';

    @attribute({
        label: t`Module ID`,
        validation: yup.string(),
        autoTable: false
    })
    module_id = '';

    @attribute({
        label: t`Target ID`,
        validation: yup.string().nullable(),
        autoTable: false
    })
    target_id = '';

    @attribute({
        label: t`Action ID`,
        validation: yup.string().nullable(),
        autoTable: false
    })
    action_id = '';

    @attribute({
        label: t`Generating?`,
        validation: yup.boolean()
    })
    generating = true;

    @attribute({
        label: t`Error`,
        validation: yup.string().nullable()
    })
    generation_error?: string | null = null;

    @attribute({
        label: t`Generation Percent`,
        validation: yup.string().nullable()
    })
    generation_percent?: number | null = null;

    @attribute({
        label: t`Download Link`,
        validation: yup.string().nullable()
    })
    download_link?: string | null = null;

    @attribute({label: t`Created At`, autoForm: true})
    created_at: Date = new Date();

    @computed
    get isDone() {
        return !this.generating && !this.generation_error && (this.generation_percent === 100);
    }

    download = () => {
        if (this.isDone && this.download_link) {
            FileSaver.saveAs(this.download_link)
        }
    };

    rerun = () => {
        (this.store.transportLayer as PdfResultTransport<PdfResultModel>).rerun(this.id as string);
    }
}

export const PdfResultStore = PdfResultModel.store;
