import {attribute, model, Model, Transport} from '@rx-suite/data';
import {t} from "@lingui/macro";
import * as yup from "yup";
import {action, computed, override} from "mobx";

@model({
    transportLayer: new Transport("/api/templates", "template"),
    permissionTarget: "Template",
    routingInfo: {
        list: "/templates",
        create: "/templates/create",
        update: "/templates/:id/edit",
        read: "/templates/:id/read"
    }
})
export class TemplateModel extends Model {
    constructor() {
        super();
        super.setup();
    }

    @attribute({
        label: t`Name`,
        description: t`This is how you will identify the template in lists (for example, you can search for this name).  It will also be shown as a title on some component types.`,
        validation: yup.string().required(t`You must give your template a name`)
    })
    name = '';

    @attribute({
        label: t`Description`,
        description: t`Add any notes here.  The information will not be included in the generated PDF - it is for reference purposes only.`,
        validation: yup.string().nullable()
    })
    desc = '';

    @attribute({
        label: t`Width`,
        validation: yup.number().typeError(t`Width must be a number`).required(t`You must give your template a width`).test(
            'isGreaterThanZero?',
            t`Width must be a positive number`,
            (value) => value > 0
        )
    })
    width = 21.6;

    @attribute({
        label: t`Height`,
        validation: yup.number().typeError(t`Height must be a number`).required(t`You must give your template a height`).test(
            'isGreaterThan0?',
            t`Width must be a positive number`,
            (value) => value > 0
        )
    })
    height = 27.9;

    @attribute({
        label: t`Top Margin`,
        validation: yup.number().typeError(t`Margin must be a number`).min(0, t`Margin must be at least 0`)
    })
    margin_top = 1;

    @attribute({
        label: t`Bottom Margin`,
        validation: yup.number().typeError(t`Margin must be a number`).min(0, t`Margin must be at least 0`)
    })
    margin_bottom = 1.5;

    @attribute({
        label: t`Left Margin`,
        validation: yup.number().typeError(t`Margin must be a number`).min(0, t`Margin must be at least 0`)
    })
    margin_left = 0.5;

    @attribute({
        label: t`Right Margin`,
        validation: yup.number().typeError(t`Margin must be a number`).min(0, t`Margin must be at least 0`)
    })
    margin_right = 0.5;

    @attribute({
        label: t`Landscape`,
        validation: yup.boolean()
    })
    landscape = false;

    @attribute({
        label: t`Unit`,
        description: t`The unit of measure measurement the width, height, and margin numbers use.`,
        validation: yup.string()
    })
    length_unit = 'cm';

    @attribute({
        label: t`Canned ID`,
        validation: yup.string().nullable()
    })
    canned_id: string | null | undefined = null;

    @attribute({label: t`Created At`})
    created_at: Date = new Date();

    @attribute({label: t`Updated At`})
    updated_at: Date = new Date();

    get units() {
        return [
            {
                label: t`Centimeters`,
                value: 'cm',
                scale: 1
            },
            {
                label: t`Inches`,
                value: 'in',
                scale: 0.393701
            },
        ]
    }

    get paperTypes() {
        return [
            {
                value: 'letter',
                label: t`Letter`,
                in: {width: 8.50, height: 11.00},
                cm: {width: 21.6, height: 27.9}
            },
            {
                value: 'a4',
                label: t`A4`,
                in: {width: 8.27, height: 11.69},
                cm: {width: 21.0, height: 29.7}
            },
            {
                value: 'legal',
                label: t`Legal`,
                in: {width: 8.50, height: 14.00},
                cm: {width: 21.6, height: 35.6}
            }
        ] as unknown as [{ [s: string]: { width: number, height: number } | string }]
    }

    convert(value: number, fromUnit: string, toUnit: string) {
        const fromDef = this.units.find(u => u.value === fromUnit);
        const toDef = this.units.find(u => u.value === toUnit);
        if (fromDef && fromDef.scale && toDef && toDef.scale) {
            return Number((value * toDef.scale / fromDef.scale).toFixed(2));
        } else {
            return value;
        }
    }

    @action.bound
    setUnitType(newUnit: string) {
        const oldUnit = this.viewModel.length_unit;
        this.viewModel.length_unit = newUnit;
        this.viewModel.width = this.convert(this.viewModel.width, oldUnit, newUnit);
        this.viewModel.height = this.convert(this.viewModel.height, oldUnit, newUnit);
        this.viewModel.margin_top = this.convert(this.viewModel.margin_top, oldUnit, newUnit);
        this.viewModel.margin_left = this.convert(this.viewModel.margin_left, oldUnit, newUnit);
        this.viewModel.margin_right = this.convert(this.viewModel.margin_right, oldUnit, newUnit);
        this.viewModel.margin_bottom = this.convert(this.viewModel.margin_bottom, oldUnit, newUnit);
    }

    @computed
    get paperType() {
        const match = this.paperTypes.find(type => {
            const data = type[(this.viewModel ? this.viewModel.length_unit : this.length_unit)] as { height: number, width: number } | undefined;
            return data && (data.width.toFixed(2) === Number(this.width).toFixed(2)) && (Number(data.height).toFixed(2) === Number(this.height).toFixed(2));
        });
        return match ? match.value as string : 'custom'
    }

    @action.bound
    handleSetPaperType(newType: string | undefined) {
        const match = this.paperTypes.find(type => {
            return type.value === newType
        });
        if (match) {
            const data = match[this.viewModel.length_unit] as { height: number, width: number } | undefined;
            if (data) {
                this.viewModel.width = data.width;
                this.viewModel.height = data.height;
            } else {
                console.warn(`${JSON.stringify(match)} has now dimension data for ${this.length_unit} on ${JSON.stringify(this.toJSON())}`);
            }
        } else {
            console.warn(`Tried to set invalid paper type (${newType}) on ${JSON.stringify(this.toJSON())}`);
        }
    }

    @override
    get isUserDeletable() {
        return (!this.canned_id);
    }

    @override
    get isUserUpdatable() {
        return (!this.canned_id);
    }
}

export const TemplateStore = TemplateModel.store;
