# == Schema Information
#
# Table name: pdf_results
#
#  id                 :bigint           not null, primary key
#  generating         :boolean
#  generation_error   :string
#  generation_percent :float
#  keep_for           :integer
#  local_storage      :string
#  name               :string
#  notes              :string
#  pdf_cache_key      :string
#  sources            :string
#  created_at         :datetime         not null
#  updated_at         :datetime         not null
#  action_id          :string
#  module_id          :string
#  target_id          :string
#  template_id        :bigint
#
# Indexes
#
#  index_pdf_results_on_template_id  (template_id)
#
# Foreign Keys
#
#  fk_rails_...  (template_id => templates.id) ON DELETE => nullify
#
require "combine_pdf"

class PdfResult < ApplicationRecord
  has_one_attached :data_file
  belongs_to :template, optional: true

  class << self
    def display_name
      "PDF Result"
    end
  end

  def as_json(options = {})
    super(options.reverse_merge(methods: :download_link))
  end

  def download_link
    "/api/pdf_results/#{id}/download"
  end

  def generate(token)
    data_file.purge
    pdf = CombinePDF.new
    generator_opts = JSON.parse(self[:sources])
    wrapped_opts = (generator_opts.is_a?(Array) ? generator_opts : [generator_opts])
    wrapped_opts.each_index do |i|
      opts = wrapped_opts[i]
      tmp_pdf = make_pdf_from_url(opts["url"], token || RxModule::Authorization.current_user_token, opts["render_selector"])
      pdf << CombinePDF.load(tmp_pdf)
      File.delete(tmp_pdf)
      update(generation_percent: ((i + 1).to_f / wrapped_opts.length.to_f) * 100)
    end
    data_file.attach(io: StringIO.new(pdf.to_pdf), content_type: "application/pdf", filename: "#{self[:name] || "result"}.pdf", identify: false)
    update(generation_percent: 100)
    save
  rescue => err
    Rails.logger.error { {msg: "Failed to generate PDF", details: as_json} }
    data_file.purge
    update(generation_percent: nil, generation_error: err.message)
  ensure
    update(generating: false)
    save
  end

  def tmp_dir
    dir = Rails.root.join("storage", "tmp")
    FileUtils.mkdir_p(dir) unless File.exist?(dir)
    dir
  end

  def make_pdf_from_url(url, token, render_selector = nil)
    exe = "\"#{Rails.root.join("bin", "generator", "makePDF.exe")}\""
    args = [
      {name: "-u",
       value: url},
      {name: "-o",
       value: tmp_dir.join("#{SecureRandom.hex(6)}}.pdf")},

      {name: "--marginTop",
       value: template&.margin_top ? "#{template.margin_top}#{template.length_unit}" : nil},
      {name: "--marginBottom",
       value: template&.margin_bottom ? "#{template.margin_bottom}#{template.length_unit}" : nil},
      {name: "--marginLeft",
       value: template&.margin_left ? "#{template.margin_left}#{template.length_unit}" : nil},
      {name: "--marginRight",
       value: template&.margin_right ? "#{template.margin_right}#{template.length_unit}" : nil},

      {name: "--height",
       value: template&.height ? "#{template.height}#{template.length_unit}" : nil},
      {name: "--width",
       value: template&.width ? "#{template.width}#{template.length_unit}" : nil},
      {name: "--landscape",
       value: template&.landscape || nil},

      {name: "--renderSelector",
       value: render_selector},

      {name: "--authorization",
       value: "TOKEN #{token}"},
      {name: "--cookies",
       value: Base64.strict_encode64(JSON.generate({name: "rxtoken", value: token}))},
      {name: "--localStorage",
       value: local_storage && local_storage != "" ? Base64.strict_encode64(local_storage) : nil}
    ].select { |arg| !arg[:value].nil? }
    command = "#{exe} #{args.map { |a| "#{a[:name]} \"#{a[:value]}\"" }.join(" ")}"
    Rails.logger.debug { command }
    puts command
    stdout, stderr, status = Open3.capture3(command)
    Rails.logger.info do
      {
        command: command,
        pdf_out: stdout,
        pdf_err: stderr,
        pdf_status: status
      }
    end
    raise "#{command} failed (#{status}) (#{stdout}|#{stderr})" unless status.success?
    JSON.parse(stdout)["file"]
  end
end
